@@ -1,8 +1,11 @@
 //! State structs.
+#![deny(missing_docs)]
 
 use anchor_lang::prelude::*;
 use anchor_lang::solana_program;
+use vipers::unwrap_or_err;
 
+/// A [SmartWallet] is a multisig wallet with Timelock capabilities.
 #[account]
 #[derive(Default, Debug, PartialEq)]
 pub struct SmartWallet {
@@ -11,14 +14,19 @@ pub struct SmartWallet {
     /// Bump seed for deriving PDA seeds.
     pub bump: u8,
 
-    /// Minimum number of owner approvals needed to sign a transaction.
+    /// Minimum number of owner approvals needed to sign a [Transaction].
     pub threshold: u64,
-    /// Minimum delay between approval and execution.
+    /// Minimum delay between approval and execution, in seconds.
     pub minimum_delay: i64,
-    /// Time after the ETA until a transaction expires.
+    /// Time after the ETA until a [Transaction] expires.
     pub grace_period: i64,
 
     /// Sequence of the ownership set.
+    ///
+    /// This may be used to see if the owners on the multisig have changed
+    /// since the last time the owners were checked. This is used on
+    /// [Transaction] approval to ensure that owners cannot approve old
+    /// transactions.
     pub owner_set_seqno: u32,
     /// Total number of [Transaction]s on this [SmartWallet].
     pub num_transactions: u64,
@@ -38,8 +46,18 @@ impl SmartWallet {
             + 4 // 4 = the Vec discriminator
             + std::mem::size_of::<Pubkey>() * (max_owners as usize)
     }
+
+    /// Gets the index of the key in the owners Vec, or error
+    pub fn owner_index(&self, key: Pubkey) -> crate::Result<usize> {
+        Ok(unwrap_or_err!(
+            self.owners.iter().position(|a| *a == key),
+            InvalidOwner
+        ))
+    }
 }
 
+/// A [Transaction] is a series of instructions that may be executed
+/// by a [SmartWallet].
 #[account]
 #[derive(Debug, Default, PartialEq)]
 pub struct Transaction {
@@ -60,7 +78,10 @@ pub struct Transaction {
     pub signers: Vec<bool>,
     /// Owner set sequence number.
     pub owner_set_seqno: u32,
-    /// Estimated time transaction will be executed
+    /// Estimated time the [Transaction] will be executed.
+    ///
+    /// - If set to [crate::NO_ETA], the transaction may be executed at any time.
+    /// - Otherwise, the [Transaction] may be executed at any point after the ETA has elapsed.
     pub eta: i64,
 
     /// The account that executed the [Transaction].
@@ -77,6 +98,11 @@ impl Transaction {
             + 4 // Vec discriminator
             + (instructions.iter().map(|ix| ix.space()).sum::<usize>())
     }
+
+    /// Number of signers.
+    pub fn num_signers(&self) -> usize {
+        self.signers.iter().filter(|&did_sign| *did_sign).count()
+    }
 }
 
 /// Instruction.
@@ -137,10 +163,14 @@ impl From<TXAccountMeta> for solana_program::instruction::AccountMeta {
 }
 
 /// Type of Subaccount.
-#[derive(AnchorSerialize, AnchorDeserialize, Copy, Clone, Debug, Eq, PartialEq)]
+#[derive(
+    AnchorSerialize, AnchorDeserialize, Copy, Clone, Debug, Eq, PartialEq, PartialOrd, Ord,
+)]
 #[repr(u8)]
 pub enum SubaccountType {
+    /// Requires the normal multisig approval process.
     Derived = 0,
+    /// Any owner may sign an instruction  as this address.
     OwnerInvoker = 1,
 }
 
@@ -150,8 +180,9 @@ impl Default for SubaccountType {
     }
 }
 
+/// Mapping of a Subaccount to its [SmartWallet].
 #[account]
-#[derive(Default, Debug, PartialEq)]
+#[derive(Copy, Default, Debug, PartialEq, Eq)]
 pub struct SubaccountInfo {
     /// Smart wallet of the sub-account.
     pub smart_wallet: Pubkey,
@@ -160,3 +191,25 @@ pub struct SubaccountInfo {
     /// Index of the sub-account.
     pub index: u64,
 }
+
+/// An account which holds the data of a single [TXInstruction].
+/// Creating this allows an owner-invoker to execute a transaction
+/// with a minimal transaction size.
+#[account]
+#[derive(Default, Debug, PartialEq)]
+pub struct StagedTXInstruction {
+    /// The [SmartWallet] to execute this on.
+    pub smart_wallet: Pubkey,
+    /// The owner-invoker index.
+    pub index: u64,
+    /// Bump seed of the owner-invoker.
+    pub owner_invoker_bump: u8,
+
+    /// The owner which will execute the instruction.
+    pub owner: Pubkey,
+    /// Owner set sequence number.
+    pub owner_set_seqno: u32,
+
+    /// The instruction to execute.
+    pub ix: TXInstruction,
+}
